{  ST9720.spin

          **************************************
                    ST7920 LCD driver
          **************************************
                    Peter Jakacki 2009
          **************************************

Serial mode driver for JHD529 128x64 LCD module.
Things are still very rough at the moment.
TO DO: Modify and integrate Graphics.Spin
}
con
  _CLKMODE = XTAL1 + PLL16X
  _XINFREQ = 5_000_000
  clockfreq = ((_CLKMODE - XTAL1) >> 6) * _XINFREQ
  _1mS  = clockfreq /     1_000 'Divisor for  1 mS

   scl       = 28
   sda       = 29
   txd       = 30               ' console I/O
   rxd       = 31               ' console I/O

  sdo     = 0
  sck     = 1
  scs     = 2

  rows  = 64
  cols  = 128

var
  long  bufwr,col,row
  byte  screen[1024]

pub main | ch,i,x,y
  cognew(@st7920,@screen)
  repeat
    bytefill(@screen,0,1024)
    ms(1000)
    'draw some lines
    repeat y from 1 to 8
      repeat x from 0 to cols
        plot(x, x/y)
    box(4,4,120,56)
    box(5,5,118,54)
    repeat i from 10 to 40 step 4
      box(i,i,20,20)
    ms(3000)

pri hline(x,y,size)
  repeat size
    plot(x++,y)

pri vline(x,y,size)
  repeat size
    plot(x,y++)

pri box(x,y,xsize,ysize)
  hline(x,y,xsize)
  vline(x,y,ysize)
  hline(x,y+ysize,xsize)
  vline(x+xsize,y,ysize)


pri plot(x,y) | i,c

  if x => 0 and x < cols and y => 0 and y < rows
    c := 7 - x&7
    screen[y << 4 + x >> 3] |= |< c

PUB ms(Period)                                    '1 mS
  if period
    waitcnt(_1mS * Period + cnt)


DAT
'
' The LCD is placed into graphics mode and updated continuously from the buffer so that
' it behaves similar to any refreshed display such as a TV or VGA
' The buffer size is 1K mapped from start to end over to top left to right, top to bottom on the LCD
' The msb of a byte is left-most.

                        org

ST7920                  andn    outa,_load
                        andn    outa,_sck
                        or      dira,_load
                        or      dira,_sck
                        or      dira,_sdo
                        mov     udly,cnt
                        add     udly,_udly

                        mov     r0,#$01
                        call    #txcmdl
                        mov     r0,#$06
                        call    #txcmd
                        mov     r0,#$0C
                        call    #txcmd
                        mov     r0,#$18
                        call    #txcmd
                        mov     r0,#$24
                        call    #txcmd
                        mov     r0,#$22
                        call    #txcmd
                        mov     r0,#$02
                        call    #txcmd

                        mov     r0,#$36
                        call    #txcmd

lcd_loop
                        mov     bufptr,par      ' set start address
                        mov     yptr,#0
lcd_loopy               mov     r0,yptr           ' read line
                        or      r0,#$80         ' Y addr
                        call    #txcmd
                        mov     r0,#$80         ' X = 0 (start of line)
                        call    #txcmd
                        mov     xcnt,#16          ' 16 bytes per line
lcd_loopx               rdbyte  r0,bufptr
                        call    #txdat
                        add     bufptr,#1
                        djnz    xcnt,#lcd_loopx
                        add     yptr,#1
                        cmp     yptr,#32 wz
        if_ne           jmp     #lcd_loopy   ' next line
'
                        mov     yptr,#0
lcd_loopy1              mov     r0,yptr           ' read line
                        or      r0,#$80         ' Y addr
                        call    #txcmd
                        mov     r0,#$88         ' X = 8 (start of line in 2nd half)
                        call    #txcmd
                        mov     xcnt,#16          ' 16 bytes per line
lcd_loopx1              rdbyte  r0,bufptr
                        call    #txdat
                        add     bufptr,#1
                        djnz    xcnt,#lcd_loopx1
                        add     yptr,#1
                        cmp     yptr,#32 wz
        if_ne           jmp     #lcd_loopy1   ' next line
                        mov     udly,cnt
                        add     udly,ms10
                        waitcnt udly,#0
                        jmp     #lcd_loop


txdat                   mov     rssel,rsmask
                        call    #transmit
                        mov     udly,cnt
                        add     udly,lcddly
                        waitcnt udly,#0
txdat_ret               ret

txcmd                   mov     rssel,#0
                        call    #transmit
                        mov     udly,cnt
                        add     udly,lcddly
                        waitcnt udly,#0
txcmd_ret               ret

txcmdl                  mov     rssel,#0
                        call    #transmit
                        mov     udly,cnt
                        add     udly,ms1
                        add     udly,ms1
                        waitcnt udly,#0
txcmdl_ret              ret

transmit                ' transmit r0 as 25 bits
                        mov     r1,r0
                        and     r1,umask        ' d7d6d5d4
                        rol     r1,#8
                        rol     r0,#4
                        and     r0,#$f0         ' d3..d0
                        or      r0,r1
                        or      r0,synch
                        or      r0,rssel
                        or      outa,_load
                        rol     r0,#8           ' prep data shift from bit23
                        mov     r2,#24
:txloop                 rol     r0,#1  wc
                        muxc    outa,_sdo
                        nop
                        nop
                        or      outa,_sck
                        nop
                        nop
                        andn    outa,_sck
                        nop
                        nop
                        djnz    r2,#:txloop
                        andn    outa,_load
transmit_ret            ret

_load                   long |<scs
_sdo                    long |<sdo
_sck                    long |<sck
msb                     long $80000000
synch                   long %11111000_00000000_00000000
_udly                   long 80000
umask                   long %1011110000
rsmask                  long %00000010_00000000_00000000
ms1                     long 80000
ms10                    long 800000
lcddly                  long 80*72

r0                      res  1
r1                      res  1
r2                      res  1
r3                      res  1
r4                      res  1
r5                      res  1
yptr                    res  1
xcnt                    res  1
bufptr                  res  1
rssel                   res  1
udly                    res  1


